import { Page, Locator } from "@playwright/test"
import { initialize_with_undefined_values, setCredentialValues } from "../steps";

export class ViewCredentialPage {

    readonly page: Page
    readonly primaryTitle: Locator
    readonly secondaryTitle: Locator

    constructor(page: Page) {
        this.page = page;
        this.primaryTitle = page.getByRole('heading', { name: 'My wallet' });
        this.secondaryTitle = page.getByRole('heading', { name: ' Credential' });
    }
    async getPrimaryTitle() {
        try {
          return await this.primaryTitle.innerText();
        } catch (error) {
          console.error("Failed to get primary title:", error);
          throw error;
        }
      }
    
      async getSecondaryTitle() {
        try {
          return await this.secondaryTitle.innerText();
        } catch (error) {
          console.error("Failed to get secondary title:", error);
          throw error;
        }
      }

    async buildACredentialFromInputValues(credentialType: string) {

        const elementsWithLabel = await this.page.locator('.col-3 strong').all();
        const elementsWithValue = await this.page.locator('.col.bg-light.text-secondary').all();
        let credential = initialize_with_undefined_values(credentialType);

        let labels: string[] = [];
        let values: string[] = [];

        for (let i = 0; i < elementsWithLabel.length; i++) {
            let label = await elementsWithLabel[i].textContent()
            if (label) label = label.trim().replace(/:$/, '');
            let value = await elementsWithValue[i].textContent()
            if (value) value = value.trim();
            if (value != null && label != null && label != 'Id' && label != "Issuance date" && label != "Status") {
                labels.push(label);
                values.push(value);
            }
        }

        credential = setCredentialValues(labels, values, credentialType);
        return credential;

    }

}
