import { expect, Page } from '@playwright/test'

import { appendRandomNumberToFilename, copyFile } from './utils'

import { CREDENTIAL_TYPE_DATASTORE_UNDEFINED, CREDENTIAL_TYPES_DATA_STORE } from './data_stores/credentials_data_store'
import { FinancialVulnerabilityCredential_fields } from './interfaces/credential_interfaces'
import { CREDENTIAL_TYPES_FIELDS_LIST, CREDENTIAL_TYPES_REQUIRED_FIELDS_LIST } from './constants/credential_fields'
import { LogInPage } from './page-objects/COMM_LoginPage'
import { ADMIN_EMAIL, ADMIN_K, ENCRYPTION_KEY, USER_K } from './constants/env_constants'
import { LeftMenuAdminPage } from './page-objects/AD_LeftMenuAdminPage'
import { LeftMenuUserPage } from './page-objects/US_LeftMenuUserPage'
import { TemplatesPage } from './page-objects/AD_TemplatesPage'
import { ViewUsersPage } from './page-objects/AD_ViewUsersPage'
import { ViewMyCredentialsPage } from './page-objects/US_ViewMyCredentialsPage'
import { UserPersonalInformationPage } from './page-objects/AD_UserPersonalInformationPage'
import { ImportTemplatePage } from './page-objects/AD_ImportTemplatePage'
import { ViewImportedDataPage } from './page-objects/AD_ViewImportedDataPage'
import { User } from './interfaces/User'
import { EncryptionKeyPage } from './page-objects/AD_EncryptionKeyPage'
import { DataProtectionPage } from './page-objects/COMM_DataProtectionPage'


export async function loginAsAdmin(page: Page, url: string) {
    try {
        const loginPage = new LogInPage(page);
        const encryptionKeyPage = new EncryptionKeyPage(page);
        const dataProtectionPage = new DataProtectionPage(page);

        await loginPage.visit(url);
        await loginPage.login(ADMIN_EMAIL, ADMIN_K);

        let currentTitle = await page.title();
        console.log("current title: ", currentTitle);

        if (currentTitle === 'Encryption Key – IdHub') {
            //code to set Encription Key
            await encryptionKeyPage.enterKey(ENCRYPTION_KEY);
            await encryptionKeyPage.clickSaveButton();
          
            await expect(page).toHaveTitle('Data protection – IdHub');
            currentTitle = await page.title();
        }

        if (currentTitle === 'Data protection – IdHub') {
            // Code to accept terms and conditions
            await dataProtectionPage.checkAcceptPrivacyCheckBox();
            await dataProtectionPage.checkAcceptCookiesCheckBox();
            await dataProtectionPage.checkAcceptLegalCheckBox();
            await dataProtectionPage.clickConfirmButton();
        }
        await expect(page).toHaveTitle('Dashboard – IdHub');

    } catch (error) {
        console.error(`Failed to login as admin: `);
        throw error;
    }
}

export async function loginAsUser(page: Page, userEmail: string, url: string) {

    try {
        const loginPage = new LogInPage(page);
        const dataProtectionPage = new DataProtectionPage(page);

        await loginPage.visit(url);
        await loginPage.login(userEmail, USER_K);

        const currentTitle = await page.title();

        if (currentTitle === 'Data Protection – IdHub') {
            // Code to accept terms and conditions
            await dataProtectionPage.checkAcceptPrivacyCheckBox();
            await dataProtectionPage.checkAcceptCookiesCheckBox();
            await dataProtectionPage.checkAcceptLegalCheckBox();
            await dataProtectionPage.clickConfirmButton();
        }
        await expect(page).toHaveTitle('Dashboard – IdHub');
    } catch (error) {
        console.error(`Failed to login as user: `);
        throw error;
    }
}

export async function clickDashboardOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const dashboardLink = await leftMenuPage.getDashboardLink();
        await dashboardLink.click({ timeout: 60000 });
        return { dashboardLink }
    } catch (error) {
        console.error(`Failed to access Dashboard from left menu: `);
        throw error;
    }
}

export async function clickUsersOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const usersLink = await leftMenuPage.getUsersLink();
        await usersLink.click()
        return { usersLink }
    } catch (error) {
        console.error(`Failed to access Users option on left menu: `);
        throw error;
    }
}
export async function clickViewUsersOnLeftMenu(page: Page) {
    try {
        clickUsersOnLeftMenu(page);
        const leftMenuPage = new LeftMenuAdminPage(page);
        const viewUsersLink = await leftMenuPage.getViewUsersLink();
        await viewUsersLink.click();
        return { viewUsersLink }
    } catch (error) {
        console.error(`Failed to access View Users Option on left menu: `);
        throw error;
    }
}
export async function clickAddUserOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const addUserLink = await leftMenuPage.getAddUserLink();
        await addUserLink.click();
        return { addUserLink }
    } catch (error) {
        console.error(`Failed to access Add Users option on left menu: `);
        throw error;
    }
}

export async function clickMyCredentialsOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuUserPage(page);
        const myCredentialsLink = await leftMenuPage.getMyCredentialsLink();
        await myCredentialsLink.click();
        return { myCredentialsLink }
    } catch (error) {
        console.error(`Failed to access My Credential option on left menu: `);
        throw error;
    }
}

export async function clickManageRolesOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const manageRolesLink = await leftMenuPage.getManageRolesLink();
        await manageRolesLink.click()
        return { manageRolesLink }
    } catch (error) {
        console.error(`Failed to access Manage Roles option on left menu: `);
        throw error;
    }
}

export async function clickManageServicesOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const manageRolesLink = await leftMenuPage.getManageServicesLink();
        await manageRolesLink.click()
        return { manageRolesLink }
    } catch (error) {
        console.error(`Failed to access Manage Services option on left menu: `);
        throw error;
    }
}

export async function clickTemplatesOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const templatesLink = await leftMenuPage.getTemplatesLink();
        await templatesLink.click()
        return { templatesLink }
    } catch (error) {
        console.error(`Failed to access Templates option on left menu: `);
        throw error;
    }
}

export async function clickDataOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const dataLink = await leftMenuPage.getDataLink();
        await dataLink.click()
        return { dataLink }
    } catch (error) {
        console.error(`Failed access Data on left menu: `);
        throw error;
    }
}

export async function clickCredentialsOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const credentialsLink = await leftMenuPage.getCredentialsLink();
        await credentialsLink.click()
        return { credentialsLink }
    } catch (error) {
        console.error(`Failed to access Credentials On left Menu: `);
        throw error;
    }
}

export async function clickViewCredentialsOnLeftMenu(page: Page) {
    try {
        const leftMenuPage = new LeftMenuAdminPage(page);
        const viewCredentialsLink = await leftMenuPage.getViewCredentialsLink();
        await viewCredentialsLink.click()
        return { viewCredentialsLink }
    } catch (error) {
        console.error(`Failed to access View Credential on left menu: `);
        throw error;
    }
}

export async function clickViewSchemaOnLeftMenu(page: Page, templateName: string) {
    try {
        const templatesPage = new TemplatesPage(page)
        //Access the specific template schema
        const { templatesLink } = await clickTemplatesOnLeftMenu(page)
        expect(await templatesPage.schemaIsAvailableInView(templateName)).toBeTruthy()
        await templatesPage.gotoViewSchemaPage(templateName)
    } catch (error) {
        console.error(`Failed to access the Template option on left menu : `);
        throw error;
    }
}


export async function gotoBasicInfoPageOfTheUser(page: Page, user: User) {

    const viewUsersPage = new ViewUsersPage(page)
    //Access the specific UPI
    const { viewUsersLink } = await clickViewUsersOnLeftMenu(page)
    try {
        expect(await viewUsersPage.userExists(user.email)).toBeTruthy()
    } catch (error) {
        console.error('Failed validating that user exists', error);
    }
    await viewUsersPage.gotoUserInformationPage(user)
}

export async function gotoViewEnabledCredential(page: Page, credentialType: string) {
    const myCredentialsPage = new ViewMyCredentialsPage(page)

    //Access the specific enabled credential that is expected to be in the user wallet!
    const { myCredentialsLink } = await clickMyCredentialsOnLeftMenu(page);
    try {
        expect(await myCredentialsPage.enabledCredentialExistInMyCredentials(credentialType)).toBeTruthy();
    } catch (error) {
        console.error('Failed accessing the enabled credential of type ${credentialType}', error);
    }
    await myCredentialsPage.gotoViewEnabledCredentialPage(credentialType);
}

export async function checkIfTheInformationIsValid(page: Page, user: User) {
    console.log(user.firstName, "membership: ", user.membershipType, "sd: ", user.startDate)
    const userPersonalInformationPage = new UserPersonalInformationPage(page)

    try {
        expect(await userPersonalInformationPage.isTheUserInfoValid(user)).toBeTruthy();
    } catch (error) {
        console.error('Failed validating the user information:', error);
    }
}


export function expectedCredentialSubjectForUser<T extends FinancialVulnerabilityCredential_fields, MembershipCard_fields>(email: string, credentialType: string): FinancialVulnerabilityCredential_fields | MembershipCard_fields | undefined {
    try {
        const testingDataForCredential = CREDENTIAL_TYPES_DATA_STORE[credentialType + '_data'] as T[];

        if (!testingDataForCredential) {
            console.error(`No testing data found for credential type: ${credentialType}`);
            return undefined;
        }

        console.log("testing Data:  ", testingDataForCredential);
        let credentialSubject = testingDataForCredential.find(user => user.email.toLowerCase() === email.toLowerCase());
        return credentialSubject;
    } catch (error) {
        console.error('Failed retriving the expected credential:', error);
    }
}

export async function importSchema(page: Page, jsonSchema: string) {
    try {
        const templatesPage = new TemplatesPage(page);
        const importTemplatePage = new ImportTemplatePage(page)
        await templatesPage.gotoAddTemplatePage();
        expect(importTemplatePage.schemaIsAvailableToImport(jsonSchema)).toBeTruthy();
        await importTemplatePage.gotoImportTemplate(jsonSchema);
        return await templatesPage.schemaIsAvailableInView(jsonSchema);
    } catch (error) {
        console.error('Failed to import the schema:', error);
    }
}


export async function checkFileName(page: Page, fileName: string): Promise<string> {
    const viewImportedDataPage = new ViewImportedDataPage(page);
    let fileInTheList = await viewImportedDataPage.fileIsAvailableInView(fileName);
    if (fileInTheList) {
        let newName = appendRandomNumberToFilename(fileName);
        //rename the file
        copyFile(fileName, newName);
        return newName;
    }
    return fileName;
}

export async function loadIfJsonSchemaNotAvailable(page: Page, jsonSchema: string) {
    try {
        //Ensure that the json schema associated with the xls is available
        const templatesPage = new TemplatesPage(page);
        await clickTemplatesOnLeftMenu(page);
        //check if the schema is available
        let schemaAvailable = await templatesPage.schemaIsAvailableInView(jsonSchema);
        if (!schemaAvailable) {
            //import the schema
            expect(await importSchema(page, jsonSchema)).toBeTruthy();
        }
    } catch (error) {
        console.error('Failed to load schema:', error);
    }
}

export function setCredentialValues(labels: string[], values: string[], credentialType: string): any {
    // Get the interface type associated with the credentialType

    const listOfFields = CREDENTIAL_TYPES_FIELDS_LIST[credentialType];
    const listOfRequiredFields = CREDENTIAL_TYPES_REQUIRED_FIELDS_LIST[credentialType];

    // Initialize an empty object to store the credential
    let credential: any = {};
    try {
        // Iterate over labels and values
        for (let i = 0; i < labels.length; i++) {
            let label = labels[i];
            let value = values[i];

            // Check if the label exists in the interface type
            label = label.charAt(0).toLowerCase() + label.slice(1);
            if (listOfFields.includes(label)) {
                credential[label] = value;
            } else {
                // If the label does not exist, add it as "unknown_data"
                credential[label] = "unknown_data";
            }
        }
        // Return the credential object
        return credential;
    } catch (error) {
        console.error('Failed to set credential values:', error);
    }
}

export function initialize_with_undefined_values(credentialType: string) {
    return (CREDENTIAL_TYPE_DATASTORE_UNDEFINED[credentialType + '_data_undefined'])
}